"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.ExportBuilder = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _provideObjectProperty = require("../util/parse/provideObjectProperty");

var _provideObjectProperty2 = _interopRequireDefault(_provideObjectProperty);

var _provideRootObject = require("../util/parse/provideRootObject");

var _provideRootObject2 = _interopRequireDefault(_provideRootObject);

var _provideMemberObject = require("../util/parse/provideMemberObject");

var _provideMemberObject2 = _interopRequireDefault(_provideMemberObject);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Class which build exports tree. This class
 * stores the information abour export clauses
 * to the internal state and build object which 
 * contains information about matches between
 * aliases and dependencies.
 * @author Dmitry Mikhaylenko
 */
var ExportBuilder = exports.ExportBuilder = function () {
    /**
     * @constructs ExportBuilder initializes the empty
     * internal state 
     */
    function ExportBuilder() {
        _classCallCheck(this, ExportBuilder);

        this.exports = {};
        this.current = undefined;
    }

    /**
     * Stores information about module which contains the exported dependencies
     * @param {string} from specifies the module from which the dependencies will
     * be exported. If the dependencies will be exported from the current module 
     * then it must have "*" value
     * @returns {ExportBuilder} self object
     * @throws {ReferenceError} if the from argument has null or undefined value 
     * or empty string
     * @author Dmitry Mikhaylenko
     */


    _createClass(ExportBuilder, [{
        key: "export",
        value: function _export(from) {
            var _this = this;

            (0, _provideRootObject2.default)(this.exports, from, function (exp) {
                _this.current = from;
                _this.exports[from] = exp;
            });

            return this;
        }

        /**
         * Stores information about exported dependency to the internal state
         * @param {string} local specifies the name of element which will be exported from the 
         * module which was specified by last invocation of export method
         * @param {string} alias specifies the name of exported element which matches with local
         * and described dependency will be exported kind of this name
         * @returns {ExportBuilder} self object
         * @throws {ReferenceError} if the local or alias argument has null or undefined value
         * or empty string
         * @author Dmitry Mikhaylenko
         */

    }, {
        key: "exportSpecifier",
        value: function exportSpecifier(local, alias) {
            (0, _provideObjectProperty2.default)(this.exports, this.current, function (exp) {
                (0, _provideMemberObject2.default)(exp, local, function (member) {
                    exp[local] = member;
                    member.aliases.push(alias);
                });
            });
            return this;
        }

        /**
         * Specifies whether the module has any iexports or not
         * @returns {boolean} exports availability marker
         * @author Dmitry Mikhaylenko
         */

    }, {
        key: "hasExports",
        value: function hasExports() {
            return JSON.stringify(this.exports) !== JSON.stringify({});
        }

        /**
         * Build the exports tree using the internal state
         * @returns {Object} exports tree
         * @author Dmitry Mikhaylenko
         */

    }, {
        key: "build",
        value: function build() {
            var result = {};
            var exports = this.exports;

            var _loop = function _loop(from) {
                var locals = exports[from];

                var _loop2 = function _loop2(local) {
                    var member = locals[local];
                    member.aliases.forEach(function (alias) {
                        result[alias] = {
                            from: from,
                            local: local
                        };
                    });
                };

                for (var local in locals) {
                    _loop2(local);
                }
            };

            for (var from in exports) {
                _loop(from);
            }
            return result;
        }
    }]);

    return ExportBuilder;
}();