"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.ImportBuilder = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _provideObjectProperty = require("../util/parse/provideObjectProperty");

var _provideObjectProperty2 = _interopRequireDefault(_provideObjectProperty);

var _provideRootObject = require("../util/parse/provideRootObject");

var _provideRootObject2 = _interopRequireDefault(_provideRootObject);

var _provideMemberObject = require("../util/parse/provideMemberObject");

var _provideMemberObject2 = _interopRequireDefault(_provideMemberObject);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Class which build impots tree. This class
 * stores the information about imports clauses
 * to the internal state and build object which
 * contains information about matches between
 * aliases and dependencies and extended modules
 * @author Dmitry Mikhaylenko
 */
var ImportBuilder = exports.ImportBuilder = function () {
    /**
     * @constructs ImportBuilder initializes the empty
     *  internal state.
     */
    function ImportBuilder() {
        _classCallCheck(this, ImportBuilder);

        this.imports = {};
        this.current = undefined;
    }

    /**
     * Stores information about module which contains required dependencies
     * @param {string} from specifies module name which contains dependencies
     * @param {boolean} extended marker that specifies necesity to extends from this object
     * @returns {ImportBuilder} self object
     * @throws {ReferenceError} if the from argument has null or undefined value or empty string
     * @author Dmitry Mikhaylenko
     */


    _createClass(ImportBuilder, [{
        key: "import",
        value: function _import(from, extended) {
            var _this = this;

            (0, _provideRootObject2.default)(this.imports, from, function (requirement) {
                _this.current = from;
                _this.imports[from] = requirement;
                requirement.extended = extended;
                requirement.members = requirement.members ? requirement.members : {};
            });
            return this;
        }

        /**
         * Stores information about dependency which imported from the module which
         * was specified by last invocation of import method
         * @param {string} member 
         * @param {string} alias specifies the name in the current module by which
         * current module source code will be refer to the imported dependency
         * @returns {ImportBuilder} self object
         * @throws {ReferenceError} if the member or alias has null or undefined value
         * or empty string
         * @author Dmitry Mikhaylenko
         */

    }, {
        key: "importSpecifier",
        value: function importSpecifier(member, alias) {
            if (alias) {
                (0, _provideObjectProperty2.default)(this.imports, this.current, function (requirement) {
                    (0, _provideMemberObject2.default)(requirement.members, member, function (mem) {
                        requirement.members[member] = mem;
                        mem.aliases.push(alias);
                    });
                });
            } else {
                throw new ReferenceError("Alias must not be null or undefined");
            }
            return this;
        }

        /**
         * Stores information about default dependency which imported from the module which
         * was specified by last invocation of import method
         * @param {*} alias
         * @returns {ImportBuilder} self object
         * @throws {ReferenceError} if the alias has null or undefined value or
         * empty string
         * @author Dmitry Mikhaylenko
         */

    }, {
        key: "importDefaultSpecifier",
        value: function importDefaultSpecifier(alias) {
            return this.importSpecifier("default", alias);
        }

        /**
         * Stores information about namespace which imported from the module which
         * was specified by last invocation of import method
         * @param {*} alias
         * @returns {ImportBuilder} self object
         * @throws {ReferenceError} if the alias has null or undefined value or
         * empty string
         * @author Dmitry Mikhaylenko
         */

    }, {
        key: "importNamespaceSpecifier",
        value: function importNamespaceSpecifier(alias) {
            return this.importSpecifier("*", alias);
        }

        /**
         * Specifies whether the module has any imports or not
         * @returns {boolean} imports availability marker
         * @author Dmitry Mikhaylenko
         */

    }, {
        key: "hasImports",
        value: function hasImports() {
            return JSON.stringify(this.imports) !== JSON.stringify({});
        }

        /**
         * Build the imports tree using the internal state
         * @returns {Object} imports tree
         * @author Dmitry Mikhaylenko
         */

    }, {
        key: "build",
        value: function build() {
            var result = {
                modules: [],
                aliases: {}
            };

            var imports = this.imports;

            var _loop = function _loop(from) {
                var req = imports[from];
                if (req.extended && result.modules.indexOf(from) === -1) {
                    result.modules.push(from);
                }

                var _loop2 = function _loop2(member) {
                    var mem = req.members[member];
                    mem.aliases.forEach(function (alias) {
                        result.aliases[alias] = {
                            from: from,
                            requirement: member
                        };
                    });
                };

                for (var member in req.members) {
                    _loop2(member);
                }
            };

            for (var from in imports) {
                _loop(from);
            }
            return result;
        }
    }]);

    return ImportBuilder;
}();