"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.TreeBuilder = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _camelcase = require("camelcase");

var _camelcase2 = _interopRequireDefault(_camelcase);

var _provideSalt = require("../util/provideSalt");

var _provideSalt2 = _interopRequireDefault(_provideSalt);

var _provideModuleName = require("../util/parse/provideModuleName");

var _provideModuleName2 = _interopRequireDefault(_provideModuleName);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Class which build statement tree. This class
 * receives import and export builder, build
 * imports and exports and merge their to the
 * statement tree
 * @author Dmitry Mikhaylenko
 */
var TreeBuilder = function () {
    /**
     * @constructs TreeBuilder store the prefix
     * @param {string} bemPrefix the prefix in the import
     * which will identify the bem modules usage
     * @author Dmitry Mikhaylenko
     */
    function TreeBuilder(bemPrefix) {
        _classCallCheck(this, TreeBuilder);

        this.prefix = bemPrefix;
    }

    /**
     * Stores the import builder to the internal state
     * @param {Object} importBuilder import builder
     * @returns {TreeBuilder} self object
     * @author Dmitry Mikhaylenko
     */


    _createClass(TreeBuilder, [{
        key: "importBuilder",
        value: function importBuilder(_importBuilder) {
            this.iBuilder = _importBuilder;
            return this;
        }

        /**
         * Stores thi export builder to the internal state
         * @param {Object} exportBuilder export builder
         * @return {TreeBuilder} self object
         * @author Dmitry Mikhaylenko
         */

    }, {
        key: "exportBuilder",
        value: function exportBuilder(_exportBuilder) {
            this.eBuilder = _exportBuilder;
            return this;
        }

        /**
         * Build statement tree
         * @param {string} exportName export name for current module
         * @returns {Object} statement descriptor
         * @throws {ReferenceError} if the import or export builder is undefined
         * @author Dmitry Mikhaylenko
         */

    }, {
        key: "build",
        value: function build(exportName) {
            var _this = this;

            if (!this.iBuilder || !this.eBuilder) {
                throw new ReferenceError("Both import builder and export builder must be defined");
            } else {
                var result = {
                    exported: exportName,
                    common: {
                        deps: [],
                        reqs: []
                    },
                    bem: {
                        deps: [],
                        reqs: ["_provide"]
                    },
                    inheritances: [],
                    imports: {},
                    exports: {}
                };

                (0, _provideSalt2.default)(function (salt) {
                    if (_this.eBuilder.hasExports()) {
                        var exports = _this.eBuilder.build();
                        for (var alias in exports) {
                            var exp = exports[alias];
                            var local = exp.local;
                            if (exp.from !== "*") {
                                if (local === "*") {
                                    _this.iBuilder.import(exp.from, true);
                                    continue;
                                } else {
                                    local = local.concat(salt);
                                    _this.iBuilder.import(exp.from, false).importSpecifier(exp.local, local);
                                }
                            }
                            result.exports[alias] = local;
                        }
                    }

                    if (_this.iBuilder.hasImports()) {
                        var imports = _this.iBuilder.build();
                        var importArray = [];
                        imports.modules.forEach(function (module) {
                            importArray.push({
                                from: module
                            });
                        });

                        for (var _alias in imports.aliases) {
                            var imp = imports.aliases[_alias];
                            importArray.push({
                                from: imp.from,
                                requirement: imp.requirement,
                                alias: _alias
                            });
                        }
                        importArray.forEach(function (imp) {
                            var fromPath = void 0,
                                fromName = void 0,
                                moduleSystem = void 0;
                            if (imp.from.startsWith(_this.prefix)) {
                                fromPath = imp.from.substring(_this.prefix.length, imp.from.length);
                                fromName = "___".concat((0, _camelcase2.default)("bem-".concat(fromPath)));
                                moduleSystem = result.bem;
                            } else {
                                fromPath = imp.from;
                                fromName = (0, _camelcase2.default)((0, _provideModuleName2.default)(fromPath, function (moduleName) {
                                    return moduleName.replace(/\\/g, "/");
                                }));
                                moduleSystem = result.common;
                            }
                            fromName = fromName.concat(salt);
                            if (moduleSystem.deps.indexOf(fromPath) === -1) {
                                moduleSystem.deps.push(fromPath);
                                moduleSystem.reqs.push(fromName);
                                if (imports.modules.indexOf(imp.from) !== -1) {
                                    result.inheritances.push(fromName);
                                }
                            }

                            if (imp.alias) {
                                result.imports[imp.alias] = {
                                    member: fromName,
                                    property: imp.requirement !== "*" ? imp.requirement : undefined
                                };
                            }
                        });
                    }
                });

                result.bem.reqs.push("_self");
                result.inheritances.push("_self");
                return result;
            }
        }
    }]);

    return TreeBuilder;
}();

exports.TreeBuilder = TreeBuilder;