"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.ExportsTraversor = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _provideSalt = require("../util/provideSalt");

var _provideSalt2 = _interopRequireDefault(_provideSalt);

var _provideSourceName = require("../util/traverse/provideSourceName");

var _provideSourceName2 = _interopRequireDefault(_provideSourceName);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * This class traverses the ast tree and replace export
 * clauses to equivalent javascript code.
 * @author Dmitry Mikhaylenko
 */

var ExportsTraversor = exports.ExportsTraversor = function () {
    /**
     * @constructs ExportsTraversor
     * @param {Object} ast root AST tree
     * @param {Object} builder export builder instance
     */
    function ExportsTraversor(ast, builder) {
        _classCallCheck(this, ExportsTraversor);

        this.ast = ast;
        this.builder = builder;
    }

    /**
     * Returns the export visitors
     * @param {Object} t babel types
     * @returns {Object} visitors
     * @author Dmitry Mikhaylenko
     */


    _createClass(ExportsTraversor, [{
        key: "getVisitors",
        value: function getVisitors(t) {
            var ast = this.ast,
                builder = this.builder;
            return {
                ExportNamedDeclaration: function ExportNamedDeclaration(path) {
                    path.traverse(visitors(t, ast, path, builder, false));
                    path.remove();
                },
                ExportDefaultDeclaration: function ExportDefaultDeclaration(path) {
                    path.traverse(visitors(t, ast, path, builder, true));
                    path.remove();
                },
                ExportAllDeclaration: function ExportAllDeclaration(path) {
                    (0, _provideSourceName2.default)(path, function (from) {
                        builder.export(from).exportSpecifier("*", from);
                        path.remove();
                    });
                }
            };
        }
    }]);

    return ExportsTraversor;
}();

/** Internal function: DO NOT USE THIS */


function visitors(t, root, ast, builder, isDefault) {
    return {
        Literal: function Literal(path) {
            (0, _provideSourceName2.default)(ast, function (from) {
                if (from === "*") {
                    (0, _provideSalt2.default)(function (salt) {
                        var local = "_exported_literal_" + salt;
                        builder.export(from).exportSpecifier(local, "default");
                        root.node.body.push(t.variableDeclaration("const", [t.variableDeclarator(t.identifier(local), path.node)]));
                    });
                }
                path.stop();
            });
        },
        Expression: function Expression(path) {
            (0, _provideSourceName2.default)(ast, function (from) {
                (0, _provideSalt2.default)(function (salt) {
                    var local = "_exported_expression_" + salt;
                    builder.export(from).exportSpecifier(local, "default");
                    root.node.body.push(t.variableDeclaration("const", [t.variableDeclarator(t.identifier(local), path.node)]));
                });
                path.stop();
            });
        },
        VariableDeclaration: function VariableDeclaration(path) {
            (0, _provideSourceName2.default)(ast, function (from) {
                root.node.body.push(path.node);
                path.node.declarations.forEach(function (declarator) {
                    builder.export(from).exportSpecifier(declarator.id.name, declarator.id.name);
                });
                path.stop();
            });
        },
        FunctionDeclaration: function FunctionDeclaration(path) {
            (0, _provideSourceName2.default)(ast, function (from) {
                if (!isDefault) {
                    builder.export(from).exportSpecifier(path.node.id.name, path.node.id.name);
                    root.node.body.push(path.node);
                } else {
                    (0, _provideSalt2.default)(function (salt) {
                        var local = "_exported_def_func_" + salt;
                        builder.export(from).exportSpecifier(local, "default");
                        root.node.body.push(t.variableDeclaration("const", [t.variableDeclarator(t.identifier(local), t.functionExpression(path.node.id, path.node.params, path.node.body, path.node.generator, path.node.async))]));
                    });
                }
                path.stop();
            });
        },
        ClassDeclaration: function ClassDeclaration(path) {
            (0, _provideSourceName2.default)(ast, function (from) {
                if (!isDefault) {
                    builder.export(from).exportSpecifier(path.node.id.name, path.node.id.name);
                    root.node.body.push(path.node);
                } else {
                    (0, _provideSalt2.default)(function (salt) {
                        var local = "_exported_def_class_" + salt;
                        builder.export(from).exportSpecifier(local, "default");
                        root.node.body.push(t.variableDeclaration("const", [t.variableDeclarator(t.identifier(local), t.classExpression(path.node.id, path.node.superClass, path.node.body, []))]));
                    });
                }
                path.stop();
            });
        },
        ExportSpecifier: function ExportSpecifier(path) {
            (0, _provideSourceName2.default)(ast, function (from) {
                builder.export(from).exportSpecifier(path.node.local.name, path.node.exported.name);
                path.skip();
            });
        }
    };
}