"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _path = require("path");

/**
 * This function provides the name of current module
 * using the path to script file. If path points to
 * the directory, that specified directory name will
 * be returned. If path points to index.js file, that
 * containing file directory name will be returned. If
 * path points to script file with name different to
 * index, that file name without extention will be
 * returned.
 * @param fullname {string} path to source file
 * @returns {string} module name
 * @throws {ReferenceError} if the fullname will to
 * have null, undefined value or empty string
 * @author Dmitry Mikhaylenko
 */
exports.default = function (fullname, callback) {
    if (fullname) {
        var filename = (0, _path.basename)(fullname);
        var index = filename.indexOf(".");
        if (index === -1) {
            return callback((0, _path.basename)(fullname));
        } else {
            var name = filename.substring(0, index);
            if (name === "index") {
                return callback((0, _path.basename)((0, _path.dirname)(fullname)));
            } else {
                return callback(name);
            }
        }
    } else {
        throw new ReferenceError("File path must not be null or undefined");
    }
};

module.exports = exports["default"];