"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

/**
 * This function provides the IIFE function statement for the 
 * current module. It builds AST tree for IIFE which wraps
 * content and send it to the callback function.
 * @param {Object} t AST babel builder
 * @param {Object} imports object which stores information about imports
 * @param {Array} content AST tree which will be wrapped by IIFE
 * @param {function} callback is a function which receives statement
 * and returns any object(optional)
 * @throws {ReferenceError} if at least one of t, imports or callback 
 * arguments has one or undefined value
 * @author Dmitry Mikhaylenko
 */
exports.default = function (t, imports, content, callback) {
    if (!t || !imports || !callback) {
        throw new ReferenceError("Arguments must not have null or undefined values");
    } else {
        var args = [],
            params = [];
        for (var alias in imports) {
            var member = imports[alias];
            params.push(t.identifier(alias));
            args.push(!member.property ? t.identifier(member.member) : t.memberExpression(t.identifier(member.member), t.identifier(member.property)));
        }

        return callback(t.callExpression(t.functionExpression(null, params, t.blockStatement(content ? content : [])), args));
    }
};

module.exports = exports["default"];