"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

/**
 * This function provides the statement of inheritance of   
 * current module from other modules. It builds AST tree 
 * for generating inherinance statement.
 * @param {Object} t AST babel builder
 * @param {Array} inheritances array of inheritance modules
 * @param {Object} inheritor object which inherits modules
 * specified in inheritances
 * @param {function} callback is a function which receives statement
 * and returns any object(optional)
 * @returns {Object} the object which is returned by callback
 * @throws {ReferenceError} if at least one of the arguments has null
 *  or undefined value 
 * @author Dmitry Mikhaylenko
 */
exports.default = function (t, inheritances, inheritor, callback) {
    if (!t || !inheritances || !callback) {
        throw new ReferenceError("Arguments must not have null or undefined value");
    } else {
        var args = inheritances.map(function (inheritance) {
            return t.identifier(inheritance);
        }).concat(inheritor ? [inheritor] : []);

        args.unshift(t.objectExpression([]));
        return callback(t.callExpression(t.memberExpression(t.identifier("Object"), t.identifier("assign")), args));
    }
};

module.exports = exports["default"];