"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

/**
 * This function provides the local exports for the current
 * module. It builds AST tree for exports information and send it
 * to the callback function.
 * @param {Object} t babel types builder
 * @param {Object} exports object which stores matches between
 * alias {string} name and exported element
 * @param {function} callback is a function which receives statement
 * and returns any object(optional)
 * @returns {Object} the object which is returned by callback
 * @throws {ReferenceError} if at least one of the arguments has null
 *  or undefined value 
 * @author Dmitry Mikhaylenko
 */
exports.default = function (t, exports, callback) {
    if (!t || !exports || !callback) {
        throw new ReferenceError("Function arguments must not have null or undefined value");
    } else {
        var props = [];

        for (var alias in exports) {
            var local = exports[alias];
            props.push(t.objectProperty(t.identifier(alias), t.identifier(local)));
        }

        return callback(t.returnStatement(t.objectExpression(props)));
    }
};

module.exports = exports["default"];